#!/usr/bin/perl -w

#   Copyright (C) 2006 David G. Jacobowitz 
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

#perl2exe_include "bytes.pm"

package cmd_driver;
use Exporter 'import';
@EXPORT_OK = qw(handleCommandFile  handleCommand
                handleCommandArray loadFileToStrArray);

use strict;
use misc qw(deep_copy genTabArray);
use parseargs qw(parseArgs);
use misc2 qw(inDict func_fail);
use eqn_string qw(create_evaluable_eqn_immed replaceBackTickedLocals);
use Scalar::Util qw(looks_like_number);
use Data::Dumper;

sub loadFileToStrArray {
 my $ifname = shift;
# print "loading file $ifname\n";
 my @sa = ();
 open(DF, "< " . $ifname) or return ("fail", undef);
 while (<DF>) {
  my $il = $_;
  chomp $il;
  push(@sa, $il);
 } 
 close DF;
 return ("ok", \@sa);
}

sub preProcessCommand {
 my $dm    = shift;
 my $gb    = shift;
 my $pre_c = shift;
 my $tr    = replaceBackTickedLocals($dm,$gb,$pre_c);
 if (func_fail($tr)) { return $tr; }
 if ($tr->{cmdline} =~ /varn?expand\(.*\)/) {
  my @cary = split(/\s/,$tr->{cmdline});
  my $ncmdline = "";
  foreach my $cp (@cary) {
   if ($cp =~ /varn?expand\((.*)\)/) {
    my $expstr = $1;
    my $sense = 1;
    if ($cp =~ /^varnexpand/) {
     $sense = 0;
    } else {
     $sense = 1;
    }
    my @vars = keys %{$dm->{data}};
    my @mvars = ();
    foreach my $v (@vars) {
     if ($v =~ /$expstr/) { 
      if ($sense == 1) { push(@mvars,$v); };
     } else {
      if ($sense == 0) { push(@mvars,$v); };
     }
    }
    if (@mvars) {
     foreach my $mv (@mvars) {
      $ncmdline .= $mv . " ";
     }
    } else {
     $tr->{status} = "warn; variable expansion yielded no matches";
    }
   } else {
    $ncmdline .= $cp . " ";
   }
  }
  $tr->{cmdline} = $ncmdline;
  return $tr;
 } else {
  return $tr;
 }
}


sub handleCommandArray {
 my $commands = shift;
 my $oo       = shift;
 my $gb       = shift;
 my $dm       = shift;
 my $cmdary   = shift;
 my $r = "ok";
 while (@{$cmdary}) {
  my $pre_c = shift @{$cmdary};  
  my $tr = preProcessCommand($dm,$gb,$pre_c);
  if (func_fail($tr)) {
   $oo->disp("> " . $tr->{status} . "\n\n");
  } else {
   my $c = $tr->{cmdline};
   if ($c =~ /\w+/) {
    $oo->disp(". " . $c . "\n");
   } else {
    $oo->disp("\n");
   }
   my $a = parseArgs($c,$dm,$gb);
   $oo->debug("____ Arguments ____\n");
   $oo->debug(Dumper($a));
   my $cmd = $a->{cmd};
   # next process the commands
   if      ($cmd eq "null") {
   } elsif ($cmd eq "do") {
    my $fname = shift @{$a->{args}};
    if (looks_like_number($fname) &&
	defined($gb->{lastdir}{files}[$fname])) {
     $fname = $gb->{lastdir}{directory} . "/" . 
              $gb->{lastdir}{files}[$fname];
     $oo->disp("Do-file name expanded to $fname\n");
    }
    if (defined($fname) && (-e $fname)) {
     my ($ok,$sa) = loadFileToStrArray($fname);
     if ($ok eq "ok") {
      handleCommandArray($commands,$oo,$gb,$dm,$sa);
      $oo->disp("(do file \"$fname\" complete)\n");
     } else {
      $r = "fail; problem opening do file";
      $oo->disp("> " . $r . "\n\n");
     }
    } else {
     $r = "fail; could not find do file";
     $oo->disp("> " . $r . "\n\n");
    }
   } elsif (inDict($cmd,$commands)) {
    fireCommand($commands,$oo,$gb,$dm,$cmd,$a);
    if ($gb->{rval}{status} eq "shutdown") {
     $oo->disp("\n Goodbye!\n\n");
     $oo->done();
     exit(0);
     goto we_are_done;
    }
   } elsif ($cmd eq "if") {
    $r = handleIf($commands,$oo,$gb,$dm,$c,$cmdary);
    if ($r =~ /^fail/) {
     $oo->disp("> " . $r . "\n\n");
    }
   } elsif ($cmd eq "while") {
    $r = handleWhile($commands,$oo,$gb,$dm,$c,$cmdary);
    if ($r =~ /^fail/) {
     $oo->disp("> " . $r . "\n\n");
    }
   } elsif ($cmd eq "foreach") {
    $r = handleForeach($commands,$oo,$gb,$dm,$c,$cmdary);
    if ($r =~ /^fail/) {
     $oo->disp("> " . $r . "\n\n");
    }
   } elsif ($cmd eq "commands") {
    showCommands($commands,$oo);
   } else {
    $r = "fail; unrecognized command: $cmd";
    $oo->disp("> " . $r . "\n\n");
   }
  }
 }
 we_are_done:
 return $r;
}

sub fireCommand {
 my $commands = shift;
 my $oo       = shift;
 my $gb       = shift;
 my $dm       = shift;
 my $cmd      = shift;
 my $a        = shift;
 my $r        = {};
 my $instance = $commands->{$cmd};
 $oo->supress($a->{quietly}); # supress output if "quietly"
 if ($a->{have_bysort}) {

  my $bysortvar = $a->{bysortvar};
  if (!inDict($bysortvar,$dm->{data})) {
   $gb->{rval}{status} = "fail; bysort variable $bysortvar not found.";
   $r = $gb->{rval}{status};
   return $r;
  }
 
  $oo->disp("\n\n");
  my $lh =  genTabArray($dm,$bysortvar);
  foreach my $tabval (keys %{$lh}) {
   $oo->disp("----------------------------------------------------------------------\n");
   $oo->disp("-> $bysortvar = $tabval\n\n");
   my $ac = deep_copy($a);
   ($ac->{ifary}, 
    $ac->{forward_iftable},
    $ac->{reverse_iftable}) = ifAryAnd($ac->{ifary},
	                         $dm->{data}{$bysortvar},
				 $tabval);
   $ac->{have_if} = 1;
   $gb->{rval2} = deep_copy($gb->{rval});
   $gb->{rval} = $instance->$cmd($ac);
   $oo->debug("____ Return Struct ____\n");
   $oo->debug(Dumper($gb->{rval}));
   if (!defined($gb->{rval}{status})) {
    $gb->{rval}{status} = "_empty_return";
   }
   if (($gb->{rval}{status} =~ /error/) or 
      ($gb->{rval}{status} =~ /_empty_return/) or
      ($gb->{rval}{status} =~ /^fail/)) {
    $oo->disp("> " . $gb->{rval}{status} . "\n\n");
    $r = $gb->{rval}{status};
   }
  }
 } else {
  $gb->{rval2} = deep_copy($gb->{rval});
  $gb->{rval} = $instance->$cmd($a);
  $oo->debug("____ Return Struct ____\n");
  $oo->debug(Dumper($gb->{rval}));
  if (!defined($gb->{rval}{status})) {
   $gb->{rval}{status} = "_empty_return";
  }
  if (($gb->{rval}{status} =~ /error/) or 
     ($gb->{rval}{status} =~ /_empty_return/) or
     ($gb->{rval}{status} =~ /^fail/)) {
   $oo->disp("> " . $gb->{rval}{status} . "\n\n");
   $r = $gb->{rval}{status};
  }
 }
 $oo->supress(0); # turn of supression
 return $r;
}


sub ifAryAnd {
 my $ifa = shift;
 my $byv = shift;
 my $tv  = shift;

 my $reta = [];
 my $retb = {};
 my $retc = [];
 my $ct = 0;
 for (my $row=0;$row<@{$byv};$row++) {
  my $ifval = 1;
  if (defined($ifa->[$row])) {
   $ifval = $ifa->[$row];
  }
  if (looks_like_number($tv)) {
   $reta->[$row] = $ifval && ($byv->[$row] == $tv);
  } else {
   $reta->[$row] = $ifval && ($byv->[$row] eq $tv);
  }
  if ($reta->[$row]) {
   $retb->{$row} = $ct;
   $retc->[$ct++] = $row;
  }
 } 
 return ($reta, $retc, $retb);
}


sub handleCommandFile {
 my $commands = shift;
 my $oo       = shift;
 my $gb       = shift;
 my $dm       = shift;
 my $cmdfile  = shift;
 my $r = "ok";
 my ($fok, $sa) = loadFileToStrArray($cmdfile);
 if ($fok eq "ok") {
  $r = handleCommandArray($commands,$oo,$gb,$dm,$sa);
 }  else   {
  $r = "fail";
 };
 return $r;
}


sub handleCommand {
 my $commands = shift;
 my $oo       = shift;
 my $gb       = shift;
 my $dm       = shift;
 my $cmd      = shift;

 my @sa       = ( $cmd );
 my $ok       = handleCommandArray($commands,$oo,$gb,$dm,\@sa);
 return $ok;
}

sub slurpBlock {
 my $sa = shift;
 my @ba = ();
 my $done = "no";
 my $opencount = 1;
 while (@{$sa} && ($done ne "yes")) {
  my $s = shift @{$sa}; 
  if ($s =~ /^.*\{\s*$/) {
   $opencount++;
  } elsif ($s =~ /^\s*\}\s*$/) {
   $opencount--;
  }
  if (!$opencount) {
   $done = "yes";
  } else {
   push(@ba, $s);
  }
 }
 if ($done eq "yes") {
  return ("ok", \@ba);
 } else {
  return ("fail", undef);
 }
}

sub handleIf {
 my $commands = shift;
 my $oo       = shift;
 my $gb       = shift;
 my $dm       = shift;
 my $cmd      = shift;
 my $ca       = shift;

 # this is not part of an object. This reference to 'self' is
 # to fake out the evaluation string generated by the
 # create_evaluable_eqn_immed function, which will reference
 # a data model in $self->{dm}
 my $self =  {};
 $self->{dm} = $dm;

 my $ifstr = $cmd;
 $ifstr =~ s/^\s*if\s*//;
 $ifstr =~ s/\{\s*$//;
 my $eqn_r = create_evaluable_eqn_immed($dm,$gb,$ifstr);
 my ($sok, $sa) = slurpBlock($ca);
 if (func_fail($eqn_r)) {
  return $eqn_r;
 } elsif ($sok eq "ok") {
  my $eval_str = $eqn_r->{expression};
  my $ifval = eval($eval_str);
  if ($@) {
#  print "ifstr: $eval_str\n";
#  print "error: $@\n";
   return "fail; $@";
  }
# print "ifval $ifval eval_str $eval_str \n";
  if ($ifval) {
   my $handle_r = handleCommandArray($commands,$oo,$gb,$dm,$sa);
   return $handle_r;
  }
 } else {
  return "fail; no ending bracket to if block";
 }
 return "ok; if block not execuated";
}

sub handleWhile {
 my $commands = shift;
 my $oo       = shift;
 my $gb       = shift;
 my $dm       = shift;
 my $cmd      = shift;
 my $ca       = shift;

 # this is not part of an object. This reference to 'self' is
 # to fake out the evaluation string generated by the
 # create_evaluable_eqn_immed function, which will reference
 # a data model in $self->{dm}
 my $self =  {};
 $self->{dm} = $dm;

 my $whstr = $cmd;
 $whstr =~ s/^\s*while\s*//;
 $whstr =~ s/\{\s*$//;
 my $eqn_r = create_evaluable_eqn_immed($dm,$gb,$whstr);
 my $eval_str = $eqn_r->{expression};
 my ($sok, $sa) = slurpBlock($ca);
 if (func_fail($eqn_r)) {
  return $eqn_r;
 } elsif ($sok eq "ok") {
  my $whval = 0;
  my $done  = 0;
  while (!$done) {
   my $whval = eval($eval_str);
#   print "whval is $whval whstr is $whstr eval_str is $eval_str\n";
   if ($@) { return "fail; $@"; }
   my $sac = deep_copy($sa);
   if ($whval) {
    my $handle_r = handleCommandArray($commands,$oo,$gb,$dm,$sac);
   } else {
    $done = 1;
   }
  }
 } else {
  return "fail; no ending bracket to while block";
 }
 return "ok; while done";
}

sub handleForeach {
 my $commands = shift;
 my $oo       = shift;
 my $gb       = shift;
 my $dm       = shift;
 my $cmd      = shift;
 my $ca       = shift;

 # this is not part of an object. This reference to 'self' is
 # to fake out the evaluation string generated by the
 # create_evaluable_eqn_immed function, which will reference
 # a data model in $self->{dm}
 my $self =  {};
 $self->{dm} = $dm;

 my $festr   = $cmd;
 $festr   =~ s/^\s*foreach\s*//;
 $festr   =~ s/\s*\{\s*$//;
 my ($vname,$liststr) = split(/\[/,$festr);
 if (!defined($vname) || !($vname =~ /\w+/)) {
  return "fail; no variable name or no list supplied for foreach";
 }
 $vname =~ s/\s*//g;

 $liststr =~ s/\s*\]\s*$//;
 $liststr =~ s/\s*,\s*/ /g;
 $liststr =~ s/^\s*//;
# print "vname $vname liststr $liststr\n";
 my @list = split(/\s/,$liststr);
 if (!@list) {
  return "fail; empty list for foreach";
 }

 my ($sok, $sa) = slurpBlock($ca);
 if ($sok ne "ok") {
  return "fail; no end to foreach block";
 }
 foreach my $item (@list) {
  $dm->{locals}{$vname} = $item; 
  my @sac = ();
  foreach my $l (@{$sa}) {
   my $nl = $l;
   # my $matchstr = "`" . $vname . "`";
   # $nl =~ s/$matchstr/$item/g;
   push (@sac,$nl);
  }
  my $handle_r = handleCommandArray($commands,$oo,$gb,$dm,\@sac);
 }
 delete $dm->{locals}{$vname};

 return "ok; foreach done";
}

sub showCommands {
 my $commands = shift;
 my $oo       = shift;
 my $ct       = 1;
 my $os = "\nAvailable commands:\n  ";
 foreach my $k (sort keys %{$commands}) {
  $os .= $k . " ";
  if (!($ct % 5)) {  $os .= "\n  "; };
  $ct++;
 }
 $os .= "\n\n\n";
 $oo->disp($os);
}

1;

