#!/usr/bin/perl -w

#   Copyright (C) 2006 David G. Jacobowitz 
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

use fgroup;
package fgroup::bootstrap_sim;
BEGIN { @ISA = qw(fgroup); };

use Exporter 'import';
@EXPORT_OK = qw();

use misc qw(one_var_stats fixedFloat fixedFloat_p numCount min max randnorm);
use dataqual qw(gen_numquald_aa);
use fgroup::simple_stat qw(format_sum_line);
use misc2 qw(inAry setFlg tstFlg inDict func_fail);

use Statistics::Distributions qw(tdistr tprob);
=uhoh -- not avail in perl 5.10
use Math::Random::MT;
=cut
use Data::Dumper;
use Scalar::Util qw(looks_like_number);

my $DEFAULT_RSE_ITERCOUNT = 100;

sub new {
 my $proto           = shift;
 my $oo              = shift;
 my $gb_ref          = shift;
 my $dm_ref          = shift;
 my $self            = {};
 my $class           = ref($proto) || $proto;
 $self->{oo}     = $oo;
 $self->{dm}     = $dm_ref;
 $self->{gb}     = $gb_ref;
=uhoh -- not avail in perl 5.10
 $self->{rander} = Math::Random::MT->new(time);
=cut
 bless ($self,$class);
 return $self;
}

sub commands {
 my $self = shift;
 my @cmds = ("bootstrap_sim");
 return @cmds;
}

sub bootstrap_sim {
 my $self    = shift;
 my $a       = shift;

 my $r       = { status => "ok" };
 my $oo      = $self->{oo};

 my @vars = ();
 foreach my $v (keys %{$a->{vars}}) {
  if ($a->{vars}{$v} eq "exist") {
   push (@vars,$v);
  } else {
   return { status => "fail; unknown variable \"$v\"" };
  }
 }

 setFlg($a,"ci",1);

 if (tstFlg($a,"noci")) {
  setFlg($a,"ci",0);
 }

 if (!@vars) {
  @vars = keys %{$self->{dm}{data}};
 }

 my $qha = {};
 foreach my $v (@vars) {
  my $qaa = gen_numquald_aa($self->{dm},[ $v ],$a->{ifary});
  $qha->{data}{$v} = $qaa->[0];
 }

 my $itercount = $DEFAULT_RSE_ITERCOUNT;
 if (defined($a->{options}{itercount}{str})) {
  if (looks_like_number($a->{options}{itercount}{str})) {
   $itercount = $a->{options}{itercount}{str};
  } else {
   return { status => "fail; iteration count not provided or not number" }
  }
 }

 my $specified_drawcount = 0;
 if (defined($a->{options}{drawcount}{str})) {
  if (looks_like_number($a->{options}{drawcount}{str})) {
   $specified_drawcount = $a->{options}{drawcount}{str};
  } else {
   return { status => "fail; draw count not number" }
  }
 } 

 my $nh = {}; # our new data hash 
 my $ds = {}; # a holder for the drawcount for each variable
 foreach my $var (@vars) {
  my $obscount = @{$qha->{data}{$var}};
#  my $drawcount = int($obscount/2);
  my $drawcount = int($obscount);
  if ($specified_drawcount) {
   if ($specified_drawcount > $obscount) {
    return { status => "fail; specified draw is greater than number obs" };
   }
   $drawcount = $specified_drawcount;
  };
  $ds->{$var} = $drawcount;
#  print "DRAWCOUNT $drawcount OBSCOUNT $obscount\n";
  for (my $iter=0;$iter<$itercount;$iter++) {
   my $mean = 0;
   for (my $draw=0;$draw<$drawcount;$draw++) {
=uhoh -- not avail in Perl5.10
    $pickindex = int($self->{rander}->rand($obscount));
=cut
    $pickindex = int(rand($obscount));
    my $pick = $qha->{data}{$var}[$pickindex];
    $mean += $pick/$drawcount;
   }
   $nh->{$var}[$iter] = $mean;
  }
 }

 $r->{data} = $nh;

 my $os = sprintf("\nBootstrap sample_iterations=%d\n",$itercount);

 $os .= sprintf("\n Variable  |    Obs       Mean  Std. Err.  [%2d%% Conf  Interval]  drawsize\n",
             $self->{gb}{conf}*100);
 $os .= "-----------+-------------------------------------------------------------\n";
 $oo->disp($os);

 my $info = {};
 foreach my $var (sort @vars) {
  my $tr = one_var_stats($r->{data}{$var});
  if (func_fail($tr)) { return $tr; };
  $r->{stats}{$var} = $tr;
  $info->{$var}{type} = "numeric";
  my $os = "";
  $os .= format_sum_line($var,$self->{gb},$tr,$a);
  $os .= sprintf(" %7d\n",$ds->{$var});
  $oo->disp($os);
 }


 if (tstFlg($a,"replace")) {
  delete $self->{dm}{data};
  $self->{dm}{data} = $nh;
  $self->{dm}{info} = $info;
 }

 $oo->disp("\n");
 return $r;
}

1;
