#!/usr/bin/perl -w

#   Copyright (C) 2006 David G. Jacobowitz 
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

use fgroup;
package fgroup::infix;
BEGIN { @ISA = qw(fgroup); };

use Exporter 'import';
@EXPORT_OK = qw();

use strict;
use csv_funcs qw(read_csv);
use misc2 qw(tstFlg);
use Scalar::Util qw(looks_like_number);
use gui qw(tk_updateVars);
use filewrap qw(open_read isUrl);

sub commands {
 my $self = shift;
 my @cmds = ("infix");
 return @cmds;
}

sub dctLoad {
 my $self = shift;	
 my $f    = shift;

 my $oo     = $self->{oo};

 my ($ok,$fh) = open_read($f);
 my $r    = {};
 if (!$ok) {
  close $fh;
  return { status => "fail; could not open dct file \"$f\"" }
 }
 my $line1 = <$fh>;
 chomp $line1;
 $line1 = lc($line1);
 my @l1_a = split(/\s/,$line1);
 my $datname = "";
 if (@l1_a < 3) {
  return { status => "fail; this may not be a dictionary file" }
 }
 my $using = 0;
 if ($l1_a[2] eq "using") { $using = 1; };
 if (($l1_a[0] ne "infix") ||
     ($l1_a[1] ne "dictionary") ||
     ($using && !($l1_a[4] =~ /\{/))) {
  return { status => "fail; I don't understand this dct file. That does not mean your file is bad; more likely we just don't handle complex dct at this time" }
 } else {
  if ($using) { $datname = $l1_a[3]; };
 }

 if (isUrl($datname)) {
 } else {
  if ($using && !(-e $datname)) {
   return { status => "fail; dat file \"$datname\" not found" }
  }
 }
 my @l = ();
 my $il = <$fh>;
 while (!($il =~ /^\}/)) {
  chomp $il;
  if ($il =~ /^\*/) {
  } else {
   @l = split(/\s+|\s*\-\s*/,$il);
   if (!@l) {
    $oo->disp("infix: empty specificationl line\n");
   } else {
    if ($l[0] =~ /^\s*$/) { shift @l; };
    my $v = $l[0];
    $r->{dictspeci}{$v}{start} = $l[1];
    $r->{dictspeci}{$v}{end}   = $l[2];
    $r->{dictspeci}{$v}{width} = $l[2] - $l[1] + 1;
   }
  }
  if (eof($fh)) { goto done_with_dict; };
  $il = <$fh>;
 }
 done_with_dict:

 if ($using) {
  close $fh;
  ($ok,$fh) = open_read($datname);
  if (!$ok) {
   close $fh;
   return { status => "fail; could not open dat file \"$datname\"" }
  } 
 }

 foreach my $v (keys %{$r->{dictspeci}}) {
  $self->{dm}{info}{$v}{type} = "numeric";
 };

 my $row = 0;
 while (<$fh>) {
  my $il = $_;
  chomp $il;
  foreach my $v (keys %{$r->{dictspeci}}) {
   my $vval = substr($il,$r->{dictspeci}{$v}{start} - 1, $r->{dictspeci}{$v}{width});
   $self->{dm}{data}{$v}[$row] = $vval;
   if (!looks_like_number($vval)) { $self->{dm}{info}{$v}{type} = "categorical" };
  }
  $row++;
 }
 $r->{status} = "ok";
 $r->{obscount} = $row;
 $r->{varcount} = keys %{$r->{dictspeci}};
 $self->{gb}{dirty} = 1;
 $oo->disp(sprintf("(%d observations read, %d variables)\n",$r->{obscount},$r->{varcount}));
 return $r;
}




sub infix {
 my $self   = shift;
 my $a      = shift;

 if ($self->{gb}{dirty}) {
  return { status => "fail; No. Data would be lost. Save or clear first." }
 }

 my $r = { status => "ok" };
 my $dctname = "";
 if (!$a->{have_using}) {
  return { status => "fail; missing using val" }
 }
 $dctname = $a->{using};

 if (isUrl($dctname)) {
 } else {
  if (!(-e $dctname)) {
   return { status => "fail; dct file \"$dctname\" not found" }
  }
 }

 $r = $self->dctLoad($dctname);
 if ($self->{gb}{using_gui}) {
  tk_updateVars($self->{dm});
 }
 return $r;
};

1;

