#!/usr/bin/perl -w

#   Copyright (C) 2006 David G. Jacobowitz 
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

use fgroup;
package fgroup::localvars;
BEGIN { @ISA = qw(fgroup); };

use Exporter 'import';
@EXPORT_OK = qw();

use strict;
use misc2 qw(inAry inDict func_fail);
use eqn_string qw(create_evaluable_eqn_immed);
use safe_eval qw(safe_eval);
use Cwd;
use Scalar::Util qw(looks_like_number);


sub commands {
 my $self = shift;
 my @cmds = ("lval", "lset", "ldrop");
 return @cmds;
}


sub lval {
 my $self = shift;
 my $a    = shift;
 my $argr = $a->{args};
 my $flgr = $a->{flgs};
 my $oo   = $self->{oo};
 my @args = @{$argr};
 my $os   = "";
 if (!@args) {
  @args = keys %{$self->{dm}{locals}};
 } else {
  foreach my $a (@args) {
   if (!defined($self->{dm}{locals}{$a})) {
    return { status => "fail; User variable \"$a\" not set." };
   }
  }
 }
 if (@args) {
  $os .= "\nUser Variables:\n\n";
  foreach my $a (@args) {
   my $val = $self->{dm}{locals}{$a};
   $os .= " $a = $val\n";
  }
  $os .= "\n";
  $oo->disp($os);
 }
 return { status => "ok" };
}


sub ldrop {
 my $self = shift;
 my $a    = shift;
 my $argr = $a->{args};
 my $flgr = $a->{flgs};
 my $oo   = $self->{oo};
 my @args = @{$argr};
 my $dvar = $args[0];
 if (defined($dvar) && defined($self->{dm}{locals}{$dvar})) {
  delete $self->{dm}{locals}{$dvar};
 } else {
  return { status => "fail; variable not defined" }	
 }
 return { status => "ok" };
}

sub lset {
 my $self = shift;
 my $a    = shift;
 my $argr = $a->{args};
 my $flgr = $a->{flgs};
 my $oo   = $self->{oo};
 my @args = @{$argr};

 my $newvar = "";
 my $neweqn = "";

 if (@args < 2) {
  return { status => "fail; missing arguments" };
 } elsif (@args == 2) {
  $newvar = $args[0];
  $neweqn = $args[1];
 } elsif (@args >= 3) {
  $newvar = shift @args;
  if ($args[0] eq "=") { shift @args; };
  while (@args) { $neweqn .= shift @args;};
 } else {
   return { status => "fail; garbled command. Format: lset var val" }
 }

 my ($newval, $problem, $report);
 my $r = create_evaluable_eqn_immed($self->{dm},$self->{gb},$neweqn);
 if (func_fail($r)) {
  return $r;
 } else {
  ($newval, $problem, $report) = safe_eval($r->{expression},$self,0,0);
 }
 $self->{dm}{locals}{$newvar} = $newval;
 if ($problem) { $oo->disp($report); }
 $oo->disp(" $newvar set to $newval\n\n");
 return $self->{gb}{rval};
}

1;

